@@ -239,6 +239,12 @@ class ElementConfig(models.Model):
         (REL_CONFIG, "relationship"),
     ]
 
+    def __str__(self):
+        text = f"Config: {self.get_config_type_display()}.{self.name}"
+        if self.parent_type:
+            text += f" ({self.get_parent_type_display()}.{self.parent_name})"
+        return text
+
     config_type = models.PositiveSmallIntegerField(choices=TYPE_CHOICES)
     name = models.TextField()
     parent_type = models.PositiveSmallIntegerField(choices=PARENT_TYPE_CHOICES, null=True)
@@ -464,7 +470,10 @@ class ObjectModelMixin(models.Model):
     updated_at = models.DateTimeField(null=True, blank=True)
 
     def raw_serialize(self):
-        return {field.name: getattr(self, field.name) for field in self._meta.fields if field.name != "search_data"}
+        return {field.name: getattr(self, field.name) for field in self._meta.fields}
+
+    def serialize(self):
+        return {field.verbose_name: getattr(self, field.name) for field in self._meta.fields}
 
     class Meta:
         abstract = True
