@@ -1,5 +1,6 @@
 from collections import OrderedDict
 from copy import deepcopy
+from functools import lru_cache
 
 from django.core.exceptions import ObjectDoesNotExist
 from django.urls import reverse
@@ -9,7 +10,7 @@ from rest_framework.validators import UniqueTogetherValidator
 from urlid_graph import settings as urlid_graph_settings
 
 from .exceptions import InvalidEdgeIdException
-from .formatting import format_property_value, format_unknown_property_value
+from .formatting import format_property_value
 from .graph_db import parse_edge_id
 from .models import ObjectRepository, RelationshipConfig, RelPropertyConfig, SavedGraph
 
@@ -18,6 +19,46 @@ def _sorted_properties(props):
     return OrderedDict([(k, v) for k, v in sorted(props.items(), key=lambda p: p[0])])
 
 
+class RelationshipSerializer:
+
+    name = None  # Must be set
+
+    def __init__(self, obj):
+        self.obj = obj
+        self._data = None
+
+    @property
+    def label(self):
+        return self.name
+
+    @property
+    def data(self):
+        if self._data is None:
+            props = {}
+            for name, value in self.obj["properties"].items():
+                key_config = RelPropertyConfig.objects.filter(parent_name=edge_label, name=name).first()
+                props[key_config.label if key_config is not None else name] = value
+            self._data = _sorted_properties(props)
+        return self._data
+
+
+def subclasses(cls):
+    """Return all subclasses of a class, recursively"""
+    children = cls.__subclasses__()
+    return set(children).union(
+        set(grandchild for child in children for grandchild in subclasses(child))
+    )
+
+
+@lru_cache(maxsize=1024)
+def get_relationship_serializer(name):
+    classes = {
+        Class.name: Class for Class in subclasses(RelationshipSerializer)
+        if Class != RelationshipSerializer
+    }
+    return classes.get(name, RelationshipSerializer)
+
+
 class ConfigSerializer(serializers.Serializer):
     options = serializers.SerializerMethodField()
     endpoints = serializers.SerializerMethodField()
@@ -107,6 +148,11 @@ class EdgeSerializer(serializers.Serializer):
 
     def get_label(self, obj):
         edge_label = obj["label"]
+        Serializer = get_relationship_serializer(edge_label)
+        label = Serializer(obj).label
+        if label != edge_label:
+            return label
+
         try:
             return RelationshipConfig.objects.get_by_name(edge_label).label
         except ObjectDoesNotExist:
@@ -117,15 +163,8 @@ class DetailedEdgeSerializer(EdgeSerializer):
     properties = serializers.SerializerMethodField()
 
     def get_properties(self, obj):
-        props = {}
-        edge_label = obj["label"]
-        for name, value in obj["properties"].items():
-            try:
-                key = RelPropertyConfig.objects.get(parent_name=edge_label, name=name).label
-            except ObjectDoesNotExist:
-                key = name
-            props[key] = format_unknown_property_value(name, key, value)
-        return _sorted_properties(props)
+        Serializer = get_relationship_serializer(obj["label"])
+        return Serializer(obj).data
 
 
 class NodeSerializer(serializers.ModelSerializer):
@@ -151,6 +190,7 @@ class DetailedNodeSerializer(NodeSerializer):
         fields = NodeSerializer.Meta.fields + ["properties"]
 
     def get_properties(self, obj):
+        # TODO: unify with Model.serialize()
         props = {}
         for name, value in obj.properties.items():
             key = obj.get_label_for_property(name)
@@ -166,6 +206,7 @@ class FullPropertiesNodeSerializer(DetailedNodeSerializer):
         fields = DetailedNodeSerializer.Meta.fields + ["full_properties"]
 
     def get_full_properties(self, obj):
+        # TODO: unify with Model.serialize()
         props = {}
         for name, value in obj.full_properties.items():
             key = obj.get_label_for_property(name)
